-- ATTACH DATABASE 'msm.db' AS msm;

/* Drop Indexes */
DROP INDEX IF EXISTS IDX_MAIN_TIME;
/* Drop Tables */
DROP TABLE IF EXISTS MAIN_COORDINATE;
DROP TABLE IF EXISTS MAIN;
DROP TABLE IF EXISTS COLUMN_KEYWORD;
DROP TABLE IF EXISTS FLAG_CMD;
DROP TABLE IF EXISTS HISTORY_COMMAND;
DROP TABLE IF EXISTS HISTORY_PARAM;
DROP TABLE IF EXISTS HISTORY;
DROP TABLE IF EXISTS TABLE_KEYWORD;
/* Create Tables */
CREATE TABLE COLUMN_KEYWORD
(
	TABLE_NAME TEXT NOT NULL,
	COLUMN_NAME TEXT NOT NULL,
	KEYWORD TEXT NOT NULL,
	-- any type
	VALUE BLOB DEFAULT NULL,
	CONSTRAINT COLUMN_KEYWORD_PKEY UNIQUE (TABLE_NAME, COLUMN_NAME, KEYWORD)
);
CREATE TABLE FLAG_CMD
(
	FLAG_CMD_ID INTEGER NOT NULL PRIMARY KEY AUTOINCREMENT,
	TIME REAL NOT NULL,
	INTERVAL REAL NOT NULL,
	FLAG_TYPE TEXT NOT NULL,
	REASON TEXT NOT NULL,
	LEVEL INTEGER NOT NULL,
	SEVERITY INTEGER NOT NULL,
	-- bool
	APPLIED INTEGER NOT NULL,
	COMMAND TEXT NOT NULL
);
CREATE TABLE HISTORY
(
	HISTORY_ID INTEGER NOT NULL PRIMARY KEY AUTOINCREMENT,
	TIME REAL NOT NULL,
	OBSERVATION_ID INTEGER NOT NULL,
	MESSAGE TEXT NOT NULL,
	PRIORITY TEXT NOT NULL,
	ORIGIN TEXT NOT NULL,
	OBJECT_ID INTEGER NOT NULL,
	APPLICATION TEXT NOT NULL
);
CREATE TABLE HISTORY_COMMAND
(
	HISTORY_ID INTEGER NOT NULL,
	-- Starts with 0.
	IDX INTEGER NOT NULL,
	CLI_COMMAND TEXT NOT NULL,
	FOREIGN KEY (HISTORY_ID)
	REFERENCES HISTORY (HISTORY_ID)
);
CREATE TABLE HISTORY_PARAM
(
	HISTORY_ID INTEGER NOT NULL,
	-- Starts with 0.
	IDX INTEGER NOT NULL,
	APP_PARAMS TEXT NOT NULL,
	FOREIGN KEY (HISTORY_ID)
	REFERENCES HISTORY (HISTORY_ID)
);
CREATE TABLE MAIN
(
	MAIN_ID INTEGER NOT NULL PRIMARY KEY AUTOINCREMENT,
	TIME REAL NOT NULL,
	TIME_EXTRA_PREC REAL DEFAULT NULL,
	ANTENNA1 INTEGER NOT NULL,
	ANTENNA2 INTEGER NOT NULL,
	ANTENNA3 INTEGER DEFAULT NULL,
	FEED1 INTEGER NOT NULL,
	FEED2 INTEGER NOT NULL,
	FEED3 INTEGER DEFAULT NULL,
	DATA_DESC_ID INTEGER NOT NULL,
	PROCESSOR_ID INTEGER NOT NULL,
	PHASE_ID INTEGER DEFAULT NULL,
	FIELD_ID INTEGER NOT NULL,
	INTERVAL REAL NOT NULL,
	EXPOSURE REAL NOT NULL,
	TIME_CENTROID REAL NOT NULL,
	PULSAR_BIN INTEGER DEFAULT NULL,
	PULSAR_GATE_ID INTEGER DEFAULT NULL,
	SCAN_NUMBER INTEGER NOT NULL,
	ARRAY_ID INTEGER NOT NULL,
	OBSERVATION_ID INTEGER NOT NULL,
	STATE_ID INTEGER NOT NULL,
	-- bool
	BASELINE_REF INTEGER DEFAULT NULL,
	U REAL NOT NULL,
	V REAL NOT NULL,
	W REAL NOT NULL,
	U2 REAL DEFAULT NULL,
	V2 REAL DEFAULT NULL,
	W2 REAL DEFAULT NULL,
	-- BLOB
	DATA BLOB DEFAULT NULL,
	-- BLOB
	FLOAT_DATA BLOB DEFAULT NULL,
	-- BLOB
	VIDEO_POINT BLOB DEFAULT NULL,
	-- BLOB
	LAG_DATA BLOB DEFAULT NULL,
	-- BLOB
	SIGMA BLOB NOT NULL,
	-- BLOB
	SIGMA_SPECTRUM BLOB DEFAULT NULL,
	-- BLOB
	WEIGHT BLOB NOT NULL,
	-- BLOB
	WEIGHT_SPECTRUM BLOB DEFAULT NULL,
	-- BLOB
	FLAG BLOB NOT NULL,
	-- BLOB
	FLAG_CATEGORY BLOB NOT NULL,
	-- bool
	FLAG_ROW INTEGER NOT NULL,
	FOREIGN KEY (PROCESSOR_ID)
	REFERENCES PROCESSOR (PROCESSOR_ID),
	FOREIGN KEY (FEED1)
	REFERENCES FEED (FEED_ID),
	FOREIGN KEY (FEED3)
	REFERENCES FEED (FEED_ID),
	FOREIGN KEY (FIELD_ID)
	REFERENCES FIELD (FIELD_ID),
	FOREIGN KEY (DATA_DESC_ID)
	REFERENCES DATA_DESCRIPTION (DATA_DESC_ID),
	FOREIGN KEY (ANTENNA1)
	REFERENCES ANTENNA (ANTENNA_ID),
	FOREIGN KEY (ANTENNA3)
	REFERENCES ANTENNA (ANTENNA_ID),
	FOREIGN KEY (FEED2)
	REFERENCES FEED (FEED_ID),
	FOREIGN KEY (ANTENNA2)
	REFERENCES ANTENNA (ANTENNA_ID),
	FOREIGN KEY (STATE_ID)
	REFERENCES STATE (STATE_ID)
);
CREATE TABLE MAIN_COORDINATE
(
	MAIN_ID INTEGER NOT NULL,
	AZ REAL NOT NULL,
	EL REAL NOT NULL,
	CLON REAL NOT NULL,
	CLAT REAL NOT NULL,
	FOREIGN KEY (MAIN_ID)
	REFERENCES MAIN (MAIN_ID)
);
CREATE TABLE TABLE_KEYWORD
(
	TABLE_NAME TEXT NOT NULL,
	KEYWORD TEXT NOT NULL,
	-- any type
	VALUE BLOB DEFAULT NULL,
	CONSTRAINT TABLE_KEYWORD_PKEY UNIQUE (TABLE_NAME, KEYWORD)
);
/* Create Indexes */
CREATE INDEX IDX_MAIN_TIME ON MAIN (TIME);
